"""HF LLM Factory"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../../nbs/05_hf.llm.factory.ipynb.

# %% auto 0
__all__ = ['GenerationFactory']

# %% ../../../nbs/05_hf.llm.factory.ipynb 3
"""
Factory module
"""

from ..resolver import Resolver

from .huggingface import HFGeneration


class GenerationFactory:
    """
    Methods to create generative models.
    """

    @staticmethod
    def create(path, method='transformers', **kwargs):
        """
        Creates a new Generation instance.

        Args:
            path: model path
            method: llm framework
            kwargs: model keyword arguments
        """

        # Derive method
        method = GenerationFactory.method(path, method)

        # Hugging Face Transformers generation
        if method == "transformers":
            return HFGeneration(path, **kwargs)
        else:
            raise ValueError(f'Unknown method: {method}')

        # Resolve custom method
        return GenerationFactory.resolve(path, method, **kwargs)

    @staticmethod
    def method(path, method):
        """
        Get or derives the LLM framework.

        Args:
            path: model path
            method: llm framework

        Return:
            llm framework
        """

        if not method:
            method = "transformers"

        return method

    @staticmethod
    def resolve(path, method, **kwargs):
        """
        Attempt to resolve a custom LLM framework.

        Args:
            path: model path
            method: llm framework
            kwargs: model keyword arguments

        Returns:
            Generation instance
        """

        try:
            return Resolver()(method)(path, **kwargs)
        except Exception as e:
            raise ImportError(f"Unable to resolve generation framework: '{method}'") from e


