"""PyTorch utilities"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/05_hf.tensors.ipynb.

# %% auto 0
__all__ = ['Pipeline', 'Tensors']

# %% ../../nbs/05_hf.tensors.ipynb 3
"""
Pipeline module
"""


class Pipeline:
    """
    Base class for all Pipelines. The only interface requirement is to define a __call___ method.
    """

    def batch(self, data, size):
        """
        Splits data into separate batch sizes specified by size.

        Args:
            data: data elements
            size: batch size

        Returns:
            list of evenly sized batches with the last batch having the remaining elements
        """

        return [data[x : x + size] for x in range(0, len(data), size)]

# %% ../../nbs/05_hf.tensors.ipynb 4
"""
Tensor processing framework module
"""

import torch


class Tensors(Pipeline):
    """
    Pipeline backed by a tensor processing framework. Currently supports PyTorch.
    """

    def quantize(self, model):
        """
        Quantizes input model and returns. This only is supported for CPU devices.

        Args:
            model: torch model

        Returns:
            quantized torch model
        """

        # pylint: disable=E1101
        return torch.quantization.quantize_dynamic(model, {torch.nn.Linear}, dtype=torch.qint8)

    def tensor(self, data):
        """
        Creates a tensor array.

        Args:
            data: input data

        Returns:
            tensor
        """

        # pylint: disable=E1102
        return torch.tensor(data)

    def context(self):
        """
        Defines a context used to wrap processing with the tensor processing framework.

        Returns:
            processing context
        """

        # pylint: disable=E1101
        return torch.no_grad()

